package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxAbnormalConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxAbnormalDO;
import cn.iocoder.yudao.module.shipment.service.boxAbnormal.BoxAbnormalService;
import cn.iocoder.yudao.module.shipment.vo.boxAbnormal.BoxAbnormalBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxAbnormal.BoxAbnormalCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxAbnormal.BoxAbnormalQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxAbnormal.BoxAbnormalUpdateReqVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 出货异常记录")
@RequestMapping("/ecw/box-abnormal")
public class BoxAbnormalController {

    @Resource
    private BoxAbnormalService boxAbnormalService;

    @PostMapping("/create")
    @ApiOperation("创建出货异常记录,use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-abnormal:create')")
    public CommonResult<Long> createBoxAbnormal(@Valid @RequestBody BoxAbnormalCreateReqVO createReqVO) {
        return success(boxAbnormalService.createBoxAbnormal(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新出货异常记录")
    @PreAuthorize("@ss.hasPermission('ecw:box-abnormal:update')")
    public CommonResult<Boolean> updateBoxAbnormal(@Valid @RequestBody BoxAbnormalUpdateReqVO updateReqVO) {
        boxAbnormalService.updateBoxAbnormal(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除出货异常记录")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-abnormal:delete')")
    public CommonResult<Boolean> deleteBoxAbnormal(@RequestParam("id") Long id) {
        boxAbnormalService.deleteBoxAbnormal(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得出货异常记录")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-abnormal:query')")
    public CommonResult<BoxAbnormalBackVO> getBoxAbnormal(@RequestParam("id") Long id) {
        BoxAbnormalDO boxAbnormal = boxAbnormalService.getBoxAbnormal(id);
        return success(BoxAbnormalConvert.INSTANCE.convert(boxAbnormal));
    }

    @GetMapping("/list")
    @ApiOperation("获得出货异常记录列表, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-abnormal:query')")
    public CommonResult<List<BoxAbnormalBackVO>> getBoxAbnormalList(@Valid BoxAbnormalQueryVO query) {
        List<BoxAbnormalDO> list = boxAbnormalService.getBoxAbnormalList(query);
        return success(BoxAbnormalConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得出货异常记录分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-abnormal:query')")
    public CommonResult<PageResult<BoxAbnormalBackVO>> getBoxAbnormalPage(@Valid BoxAbnormalQueryVO query, PageVO page) {
        PageResult<BoxAbnormalDO> pageResult = boxAbnormalService.getBoxAbnormalPage(query, page);
        return success(BoxAbnormalConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出出货异常记录 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-abnormal:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxAbnormalExcel(@Valid BoxAbnormalQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxAbnormalDO> list = boxAbnormalService.getBoxAbnormalList(query);
        // 导出 Excel
        List<BoxAbnormalBackVO> datas = BoxAbnormalConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "出货异常记录.xls", "数据", BoxAbnormalBackVO.class, datas);
    }
}
