package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.boxAirFly.BoxAirFlyConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.boxAirFly.BoxAirFlyDO;
import cn.iocoder.yudao.module.shipment.service.boxAirFly.BoxAirFlyService;
import cn.iocoder.yudao.module.shipment.vo.boxAirFly.*;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 空运起飞")
@RequestMapping("/ecw/box-air-fly")
public class BoxAirFlyController {

    @Resource
    private BoxAirFlyService boxAirFlyService;

    @PostMapping("/create")
    @ApiOperation("创建起飞")
    @Idempotent(timeout = 5)
    //@PreAuthorize("@ss.hasPermission('ecw:box-air-fly:create')")
    public CommonResult<Long> createBoxAirFly(@Valid @RequestBody BoxAirFlyCreateReqVO createReqVO) {
        return success(boxAirFlyService.createBoxAirFly(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新起飞")
    @Idempotent(timeout = 5)
    //@PreAuthorize("@ss.hasPermission('ecw:box-air-fly:update')")
    public CommonResult<Boolean> updateBoxAirFly(@Valid @RequestBody BoxAirFlyUpdateReqVO updateReqVO) {
        boxAirFlyService.updateBoxAirFly(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除起飞")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-air-fly:delete')")
    public CommonResult<Boolean> deleteBoxAirFly(@RequestParam("id") Long id) {
        boxAirFlyService.deleteBoxAirFly(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得起飞")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-air-fly:query')")
    public CommonResult<BoxAirFlyBackVO> getBoxAirFly(@RequestParam("id") Long id) {
        BoxAirFlyDO boxAirFly = boxAirFlyService.getBoxAirFly(id);
        return success(BoxAirFlyConvert.INSTANCE.convert(boxAirFly));
    }

    @GetMapping("/list")
    @ApiOperation("获得起飞列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-air-fly:query')")
    public CommonResult<List<BoxAirFlyBackVO>> getBoxAirFlyList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxAirFlyDO> list = boxAirFlyService.getBoxAirFlyList(ids);
        return success(BoxAirFlyConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得起飞分页")
    //@PreAuthorize("@ss.hasPermission('ecw:box-air-fly:query')")
    public CommonResult<PageResult<BoxAirFlyBackVO>> getBoxAirFlyPage(@Valid BoxAirFlyQueryVO query, PageVO page) {
        PageResult<BoxAirFlyDO> pageResult = boxAirFlyService.getBoxAirFlyPage(query, page);
        return success(BoxAirFlyConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出起飞 Excel")
    //@PreAuthorize("@ss.hasPermission('ecw:box-air-fly:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxAirFlyExcel(@Valid BoxAirFlyQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxAirFlyDO> list = boxAirFlyService.getBoxAirFlyList(query);
        // 导出 Excel
        List<BoxAirFlyBackVO> datas = BoxAirFlyConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "起飞.xls", "数据", BoxAirFlyBackVO.class, datas);
    }
}
