package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxArrivalAirConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxArrivalAirDO;
import cn.iocoder.yudao.module.shipment.service.boxArrivalAir.BoxArrivalAirService;
import cn.iocoder.yudao.module.shipment.vo.boxArrivalAir.BoxArrivalAirBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxArrivalAir.BoxArrivalAirCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxArrivalAir.BoxArrivalAirQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxArrivalAir.BoxArrivalAirUpdateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxArrivalOrder.BoxArrivalOrderAllCreateVO;
import cn.iocoder.yudao.module.shipment.vo.boxArrivalOrder.BoxArrivalOrderCreateVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 空运到港")
@RequestMapping("/ecw/box-arrival-air")
public class BoxArrivalAirController {

    @Resource
    private BoxArrivalAirService boxArrivalAirService;

    @PostMapping("/create")
    @ApiOperation("创建空运到港， use this")
    @Idempotent(timeout = 5)
//    @PreAuthorize("@ss.hasPermission('ecw:box-arrival-air:create')")
    public CommonResult<Long> createBoxArrivalAir(@Valid @RequestBody BoxArrivalAirCreateReqVO createReqVO) {
        return success(boxArrivalAirService.createBoxArrivalAir(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新空运到港, use this")
    @Idempotent(timeout = 5)
//    @PreAuthorize("@ss.hasPermission('ecw:box-arrival-air:update')")
    public CommonResult<Boolean> updateBoxArrivalAir(@Valid @RequestBody BoxArrivalAirUpdateReqVO updateReqVO) {
        boxArrivalAirService.updateBoxArrivalAir(updateReqVO);
        return success(true);
    }

    @PostMapping("/updateOrderArrival")
    @ApiOperation("更新订单到港状态， use this")
    public CommonResult<Boolean> updateOrderArrival(@Valid @RequestBody BoxArrivalOrderCreateVO createReqVO) {
        boxArrivalAirService.updateOrderArrival(createReqVO);
        return success(true);
    }

    @PostMapping("/updateAllOrderArrival")
    @ApiOperation("更新所有订单到港状态， use this")
    public CommonResult<Boolean> updateAllOrderArrival(@Valid @RequestBody BoxArrivalOrderAllCreateVO createReqVO) {
        boxArrivalAirService.updateOrderAllArrival(createReqVO);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得空运到港")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-arrival-air:query')")
    public CommonResult<BoxArrivalAirBackVO> getBoxArrivalAir(@RequestParam("id") Long id) {
        BoxArrivalAirDO boxArrivalAir = boxArrivalAirService.getBoxArrivalAir(id);
        return success(BoxArrivalAirConvert.INSTANCE.convert(boxArrivalAir));
    }

    @GetMapping("/list")
    @ApiOperation("获得空运到港列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-arrival-air:query')")
    public CommonResult<List<BoxArrivalAirBackVO>> getBoxArrivalAirList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxArrivalAirDO> list = boxArrivalAirService.getBoxArrivalAirList(ids);
        return success(BoxArrivalAirConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得空运到港分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-arrival-air:query')")
    public CommonResult<PageResult<BoxArrivalAirBackVO>> getBoxArrivalAirPage(@Valid BoxArrivalAirQueryVO query, PageVO page) {
        PageResult<BoxArrivalAirDO> pageResult = boxArrivalAirService.getBoxArrivalAirPage(query, page);
        return success(BoxArrivalAirConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出空运到港 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-arrival-air:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxArrivalAirExcel(@Valid BoxArrivalAirQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxArrivalAirDO> list = boxArrivalAirService.getBoxArrivalAirList(query);
        // 导出 Excel
        List<BoxArrivalAirBackVO> datas = BoxArrivalAirConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "空运到港.xls", "数据", BoxArrivalAirBackVO.class, datas);
    }
}
