package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxArrivalConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxArrivalDO;
import cn.iocoder.yudao.module.shipment.service.boxArrival.BoxArrivalService;
import cn.iocoder.yudao.module.shipment.vo.boxArrival.BoxArrivalBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxArrival.BoxArrivalCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxArrival.BoxArrivalQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxArrival.BoxArrivalUpdateReqVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 到港")
@RequestMapping("/ecw/box-arrival")
public class BoxArrivalController {

    @Resource
    private BoxArrivalService boxArrivalService;

    @PostMapping("/create")
    @ApiOperation("创建到港，use this")
    @Idempotent(timeout = 5)
//    @PreAuthorize("@ss.hasPermission('ecw:box-arrival:create')")
    public CommonResult<Long> createBoxArrival(@Valid @RequestBody BoxArrivalCreateReqVO createReqVO) {
        return success(boxArrivalService.createBoxArrival(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新到港，use this")
    @Idempotent(timeout = 5)
//    @PreAuthorize("@ss.hasPermission('ecw:box-arrival:update')")
    public CommonResult<Boolean> updateBoxArrival(@Valid @RequestBody BoxArrivalUpdateReqVO updateReqVO) {
        boxArrivalService.updateBoxArrival(updateReqVO);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得到港")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:box-arrival:query')")
    public CommonResult<BoxArrivalBackVO> getBoxArrival(@RequestParam("id") Long id) {
        BoxArrivalDO boxArrival = boxArrivalService.getBoxArrival(id);
        return success(BoxArrivalConvert.INSTANCE.convert(boxArrival));
    }

    @GetMapping("/list")
    @ApiOperation("获得到港列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-arrival:query')")
    public CommonResult<List<BoxArrivalBackVO>> getBoxArrivalList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxArrivalDO> list = boxArrivalService.getBoxArrivalList(ids);
        return success(BoxArrivalConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得到港分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-arrival:query')")
    public CommonResult<PageResult<BoxArrivalBackVO>> getBoxArrivalPage(@Valid BoxArrivalQueryVO query, PageVO page) {
        PageResult<BoxArrivalDO> pageResult = boxArrivalService.getBoxArrivalPage(query, page);
        return success(BoxArrivalConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出到港 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-arrival:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxArrivalExcel(@Valid BoxArrivalQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxArrivalDO> list = boxArrivalService.getBoxArrivalList(query);
        // 导出 Excel
        List<BoxArrivalBackVO> datas = BoxArrivalConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "到港.xls", "数据", BoxArrivalBackVO.class, datas);
    }
}
