package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxBargeConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxBargeDO;
import cn.iocoder.yudao.module.shipment.service.boxBarge.BoxBargeService;
import cn.iocoder.yudao.module.shipment.vo.boxBarge.BoxBargeBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxBarge.BoxBargeCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxBarge.BoxBargeQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxBarge.BoxBargeUpdateReqVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 驳船")
@RequestMapping("/ecw/box-barge")
public class BoxBargeController {

    @Resource
    private BoxBargeService boxBargeService;

    @PostMapping("/create")
    @ApiOperation("创建驳船, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-barge:create')")
    public CommonResult<Long> createBoxBarge(@Valid @RequestBody BoxBargeCreateReqVO createReqVO) {
        return success(boxBargeService.createBoxBarge(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新驳船, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-barge:update')")
    public CommonResult<Boolean> updateBoxBarge(@Valid @RequestBody BoxBargeUpdateReqVO updateReqVO) {
        boxBargeService.updateBoxBarge(updateReqVO);
        return success(true);
    }



    @GetMapping("/get")
    @ApiOperation("获得驳船")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-barge:query')")
    public CommonResult<BoxBargeBackVO> getBoxBarge(@RequestParam("id") Long id) {
        BoxBargeDO boxBarge = boxBargeService.getBoxBarge(id);
        return success(BoxBargeConvert.INSTANCE.convert(boxBarge));
    }

    @GetMapping("/list")
    @ApiOperation("获得驳船列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-barge:query')")
    public CommonResult<List<BoxBargeBackVO>> getBoxBargeList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxBargeDO> list = boxBargeService.getBoxBargeList(ids);
        return success(BoxBargeConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得驳船分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-barge:query')")
    public CommonResult<PageResult<BoxBargeBackVO>> getBoxBargePage(@Valid BoxBargeQueryVO query, PageVO page) {
        PageResult<BoxBargeDO> pageResult = boxBargeService.getBoxBargePage(query, page);
        return success(BoxBargeConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出驳船 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-barge:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxBargeExcel(@Valid BoxBargeQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxBargeDO> list = boxBargeService.getBoxBargeList(query);
        // 导出 Excel
        List<BoxBargeBackVO> datas = BoxBargeConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "驳船.xls", "数据", BoxBargeBackVO.class, datas);
    }
}
