package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import cn.iocoder.yudao.module.shipment.vo.boxBookAir.*;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxBookAirDO;
import cn.iocoder.yudao.module.shipment.convert.BoxBookAirConvert;
import cn.iocoder.yudao.module.shipment.service.boxBookAir.BoxBookAirService;

@Validated
@RestController
@Api(tags = "管理后台 - 订舱-空运")
@RequestMapping("/shipment/box-book-air")
public class BoxBookAirController {

    @Resource
    private BoxBookAirService boxBookAirService;

    @PostMapping("/create")
    @ApiOperation("创建订舱-空运")
    //    @PreAuthorize("@ss.hasPermission('shipment:box-book-air:create')")
    public CommonResult<Long> createBoxBookAir(@Valid @RequestBody BoxBookAirCreateReqVO createReqVO) {
        return success(boxBookAirService.createBoxBookAir(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新订舱-空运")
    //    @PreAuthorize("@ss.hasPermission('shipment:box-book-air:update')")
    public CommonResult<Boolean> updateBoxBookAir(@Valid @RequestBody BoxBookAirUpdateReqVO updateReqVO) {
        boxBookAirService.updateBoxBookAir(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除订舱-空运")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //    @PreAuthorize("@ss.hasPermission('shipment:box-book-air:delete')")
    public CommonResult<Boolean> deleteBoxBookAir(@RequestParam("id") Long id) {
        boxBookAirService.deleteBoxBookAir(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得订舱-空运")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //    @PreAuthorize("@ss.hasPermission('shipment:box-book-air:query')")
    public CommonResult<BoxBookAirBackVO> getBoxBookAir(@RequestParam("id") Long id) {
        BoxBookAirDO boxBookAir = boxBookAirService.getBoxBookAir(id);
        return success(BoxBookAirConvert.INSTANCE.convert(boxBookAir));
    }

    @GetMapping("/list")
    @ApiOperation("获得订舱-空运列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //    @PreAuthorize("@ss.hasPermission('shipment:box-book-air:query')")
    public CommonResult<List<BoxBookAirBackVO>> getBoxBookAirList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxBookAirDO> list = boxBookAirService.getBoxBookAirList(ids);
        return success(BoxBookAirConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得订舱-空运分页")
    //    @PreAuthorize("@ss.hasPermission('shipment:box-book-air:query')")
    public CommonResult<PageResult<BoxBookAirBackVO>> getBoxBookAirPage(@Valid BoxBookAirQueryVO query, PageVO page) {
        PageResult<BoxBookAirDO> pageResult = boxBookAirService.getBoxBookAirPage(query, page);
        return success(BoxBookAirConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出订舱-空运 Excel")
    //    @PreAuthorize("@ss.hasPermission('shipment:box-book-air:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxBookAirExcel(@Valid BoxBookAirQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxBookAirDO> list = boxBookAirService.getBoxBookAirList(query);
        // 导出 Excel
        List<BoxBookAirBackVO> datas = BoxBookAirConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "订舱-空运.xls", "数据", BoxBookAirBackVO.class, datas);
    }
}
