package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxBookSeaConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxBookSeaDO;
import cn.iocoder.yudao.module.shipment.service.boxBookSea.BoxBookSeaService;
import cn.iocoder.yudao.module.shipment.vo.boxBookSea.BoxBookSeaBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxBookSea.BoxBookSeaCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxBookSea.BoxBookSeaQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxBookSea.BoxBookSeaUpdateReqVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 订舱-海运")
@RequestMapping("/ecw/box-book-sea")
public class BoxBookSeaController {

    @Resource
    private BoxBookSeaService boxBookSeaService;

    @PostMapping("/create")
    @ApiOperation("添加订舱, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-book-sea:create')")
    public CommonResult<Long> createBoxBookSea(@Valid @RequestBody BoxBookSeaCreateReqVO createReqVO) {
        return success(boxBookSeaService.createBoxBookSea(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新订舱, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-book-sea:update')")
    public CommonResult<Boolean> updateBoxBookSea(@Valid @RequestBody BoxBookSeaUpdateReqVO updateReqVO) {
        boxBookSeaService.updateBoxBookSea(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除订舱")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-book-sea:delete')")
    public CommonResult<Boolean> deleteBoxBookSea(@RequestParam("id") Long id) {
        boxBookSeaService.deleteBoxBookSea(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得订舱")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-book-sea:query')")
    public CommonResult<BoxBookSeaBackVO> getBoxBookSea(@RequestParam("id") Long id) {
        BoxBookSeaDO boxBookSea = boxBookSeaService.getBoxBookSea(id);
        return success(BoxBookSeaConvert.INSTANCE.convert(boxBookSea));
    }

    @GetMapping("/list")
    @ApiOperation("获得订舱列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-book-sea:query')")
    public CommonResult<List<BoxBookSeaBackVO>> getBoxBookSeaList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxBookSeaDO> list = boxBookSeaService.getBoxBookSeaList(ids);
        return success(BoxBookSeaConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得订舱分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-book-sea:query')")
    public CommonResult<PageResult<BoxBookSeaBackVO>> getBoxBookSeaPage(@Valid BoxBookSeaQueryVO query, PageVO page) {
        PageResult<BoxBookSeaDO> pageResult = boxBookSeaService.getBoxBookSeaPage(query, page);
        return success(BoxBookSeaConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出订舱 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-book-sea:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxBookSeaExcel(@Valid BoxBookSeaQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxBookSeaDO> list = boxBookSeaService.getBoxBookSeaList(query);
        // 导出 Excel
        List<BoxBookSeaBackVO> datas = BoxBookSeaConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "订舱.xls", "数据", BoxBookSeaBackVO.class, datas);
    }
}
