package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxCabinetConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxCabinetDO;
import cn.iocoder.yudao.module.shipment.service.boxCabinet.BoxCabinetService;
import cn.iocoder.yudao.module.shipment.vo.boxCabinet.BoxCabinetBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinet.BoxCabinetCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinet.BoxCabinetQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinet.BoxCabinetUpdateReqVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 装柜")
@RequestMapping("/ecw/box-cabinet")
public class BoxCabinetController {

    @Resource
    private BoxCabinetService boxCabinetService;

    @PostMapping("/create")
    @ApiOperation("创建装柜, use this")
    @Idempotent(timeout = 5)
//    @PreAuthorize("@ss.hasPermission('ecw:box-cabinet:create')")
    public CommonResult<Long> createBoxCabinet(@Valid @RequestBody BoxCabinetCreateReqVO createReqVO) {
        return success(boxCabinetService.createBoxCabinet(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新装柜, use this")
    @Idempotent(timeout = 5)
//    @PreAuthorize("@ss.hasPermission('ecw:box-cabinet:update')")
    public CommonResult<Boolean> updateBoxCabinet(@Valid @RequestBody BoxCabinetUpdateReqVO updateReqVO) {
        boxCabinetService.updateBoxCabinet(updateReqVO);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得装柜")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-cabinet:query')")
    public CommonResult<BoxCabinetBackVO> getBoxCabinet(@RequestParam("id") Long id) {
        BoxCabinetDO boxCabinet = boxCabinetService.getBoxCabinet(id);
        return success(BoxCabinetConvert.INSTANCE.convert(boxCabinet));
    }

    @GetMapping("/list")
    @ApiOperation("获得装柜列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-cabinet:query')")
    public CommonResult<List<BoxCabinetBackVO>> getBoxCabinetList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxCabinetDO> list = boxCabinetService.getBoxCabinetList(ids);
        return success(BoxCabinetConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得装柜分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-cabinet:query')")
    public CommonResult<PageResult<BoxCabinetBackVO>> getBoxCabinetPage(@Valid BoxCabinetQueryVO query, PageVO page) {
        PageResult<BoxCabinetDO> pageResult = boxCabinetService.getBoxCabinetPage(query, page);
        return success(BoxCabinetConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出装柜 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-cabinet:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxCabinetExcel(@Valid BoxCabinetQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxCabinetDO> list = boxCabinetService.getBoxCabinetList(query);
        // 导出 Excel
        List<BoxCabinetBackVO> datas = BoxCabinetConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "装柜.xls", "数据", BoxCabinetBackVO.class, datas);
    }
}
