package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxCabinetUnloadAbnormalConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxCabinetUnloadAbnormalDO;
import cn.iocoder.yudao.module.shipment.service.boxCabinetUnloadAbnormal.BoxCabinetUnloadAbnormalService;
import cn.iocoder.yudao.module.shipment.vo.boxCabinetUnloadAbnormal.BoxCabinetUnloadAbnormalBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinetUnloadAbnormal.BoxCabinetUnloadAbnormalCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinetUnloadAbnormal.BoxCabinetUnloadAbnormalQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinetUnloadAbnormal.BoxCabinetUnloadAbnormalUpdateReqVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 卸柜异常")
@RequestMapping("/ecw/box-cabinet-unload-abnormal")
public class BoxCabinetUnloadAbnormalController {

    @Resource
    private BoxCabinetUnloadAbnormalService boxCabinetUnloadAbnormalService;

    @PostMapping("/create")
    @ApiOperation("创建卸柜异常, use this")
//    //@PreAuthorize("@ss.hasPermission('ecw:box-cabinet-unload-abnormal:create')")
    public CommonResult<Long> createBoxCabinetUnloadAbnormal(@Valid @RequestBody BoxCabinetUnloadAbnormalCreateReqVO createReqVO) {
        return success(boxCabinetUnloadAbnormalService.createBoxCabinetUnloadAbnormal(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新卸柜异常")
    //@PreAuthorize("@ss.hasPermission('ecw:box-cabinet-unload-abnormal:update')")
    public CommonResult<Boolean> updateBoxCabinetUnloadAbnormal(@Valid @RequestBody BoxCabinetUnloadAbnormalUpdateReqVO updateReqVO) {
        boxCabinetUnloadAbnormalService.updateBoxCabinetUnloadAbnormal(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除卸柜异常")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-cabinet-unload-abnormal:delete')")
    public CommonResult<Boolean> deleteBoxCabinetUnloadAbnormal(@RequestParam("id") Long id) {
        boxCabinetUnloadAbnormalService.deleteBoxCabinetUnloadAbnormal(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得卸柜异常")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-cabinet-unload-abnormal:query')")
    public CommonResult<BoxCabinetUnloadAbnormalBackVO> getBoxCabinetUnloadAbnormal(@RequestParam("id") Long id) {
        BoxCabinetUnloadAbnormalDO boxCabinetUnloadAbnormal = boxCabinetUnloadAbnormalService.getBoxCabinetUnloadAbnormal(id);
        return success(BoxCabinetUnloadAbnormalConvert.INSTANCE.convert(boxCabinetUnloadAbnormal));
    }

    @GetMapping("/list")
    @ApiOperation("获得卸柜异常列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-cabinet-unload-abnormal:query')")
    public CommonResult<List<BoxCabinetUnloadAbnormalBackVO>> getBoxCabinetUnloadAbnormalList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxCabinetUnloadAbnormalDO> list = boxCabinetUnloadAbnormalService.getBoxCabinetUnloadAbnormalList(ids);
        return success(BoxCabinetUnloadAbnormalConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得卸柜异常分页")
    //@PreAuthorize("@ss.hasPermission('ecw:box-cabinet-unload-abnormal:query')")
    public CommonResult<PageResult<BoxCabinetUnloadAbnormalBackVO>> getBoxCabinetUnloadAbnormalPage(@Valid BoxCabinetUnloadAbnormalQueryVO query, PageVO page) {
        PageResult<BoxCabinetUnloadAbnormalDO> pageResult = boxCabinetUnloadAbnormalService.getBoxCabinetUnloadAbnormalPage(query, page);
        return success(BoxCabinetUnloadAbnormalConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出卸柜异常 Excel")
    //@PreAuthorize("@ss.hasPermission('ecw:box-cabinet-unload-abnormal:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxCabinetUnloadAbnormalExcel(@Valid BoxCabinetUnloadAbnormalQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxCabinetUnloadAbnormalDO> list = boxCabinetUnloadAbnormalService.getBoxCabinetUnloadAbnormalList(query);
        // 导出 Excel
        List<BoxCabinetUnloadAbnormalBackVO> datas = BoxCabinetUnloadAbnormalConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "卸柜异常.xls", "数据", BoxCabinetUnloadAbnormalBackVO.class, datas);
    }
}
