package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxCabinetUnloadConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxCabinetUnloadDO;
import cn.iocoder.yudao.module.shipment.service.boxCabinetUnload.BoxCabinetUnloadService;
import cn.iocoder.yudao.module.shipment.vo.boxAirCheckout.BoxAirCheckoutPDAVO;
import cn.iocoder.yudao.module.shipment.vo.boxCabinetUnload.*;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 卸柜")
@RequestMapping("/ecw/box-cabinet-unload")
public class BoxCabinetUnloadController {

    @Resource
    private BoxCabinetUnloadService boxCabinetUnloadService;

    @PostMapping("/create")
    @ApiOperation("创建卸柜, use this")
    @Idempotent(timeout = 5)
//    @PreAuthorize("@ss.hasPermission('ecw:box-cabinet-unload:create')")
    public CommonResult<Long> createBoxCabinetUnload(@Valid @RequestBody BoxCabinetUnloadCreateReqVO createReqVO) {
        return success(boxCabinetUnloadService.createBoxCabinetUnload(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新卸柜, use this")
    @Idempotent(timeout = 5)
    // @PreAuthorize("@ss.hasPermission('ecw:box-cabinet-unload:update')")
    public CommonResult<Boolean> updateBoxCabinetUnload(@Valid @RequestBody BoxCabinetUnloadUpdateReqVO updateReqVO) {
        boxCabinetUnloadService.updateBoxCabinetUnload(updateReqVO);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得卸柜")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('ecw:box-cabinet-unload:query')")
    public CommonResult<BoxCabinetUnloadBackVO> getBoxCabinetUnload(@RequestParam("id") Long id) {
        BoxCabinetUnloadDO boxCabinetUnload = boxCabinetUnloadService.getBoxCabinetUnload(id);
        return success(BoxCabinetUnloadConvert.INSTANCE.convert(boxCabinetUnload));
    }

    @GetMapping("/list")
    @ApiOperation("获得卸柜列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    // @PreAuthorize("@ss.hasPermission('ecw:box-cabinet-unload:query')")
    public CommonResult<List<BoxCabinetUnloadBackVO>> getBoxCabinetUnloadList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxCabinetUnloadDO> list = boxCabinetUnloadService.getBoxCabinetUnloadList(ids);
        return success(BoxCabinetUnloadConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得卸柜分页")
    // @PreAuthorize("@ss.hasPermission('ecw:box-cabinet-unload:query')")
    public CommonResult<PageResult<BoxCabinetUnloadBackVO>> getBoxCabinetUnloadPage(@Valid BoxCabinetUnloadQueryVO query, PageVO page) {
        PageResult<BoxCabinetUnloadDO> pageResult = boxCabinetUnloadService.getBoxCabinetUnloadPage(query, page);
        return success(BoxCabinetUnloadConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出卸柜 Excel")
    // @PreAuthorize("@ss.hasPermission('ecw:box-cabinet-unload:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxCabinetUnloadExcel(@Valid BoxCabinetUnloadQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxCabinetUnloadDO> list = boxCabinetUnloadService.getBoxCabinetUnloadList(query);
        // 导出 Excel
        List<BoxCabinetUnloadBackVO> datas = BoxCabinetUnloadConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "卸柜.xls", "数据", BoxCabinetUnloadBackVO.class, datas);
    }

    @GetMapping("/getBoxCabinetUnload4PDAPage")
    @ApiOperation("PDA - 获得空运到仓分页")
    public CommonResult<PageResult<BoxCabinetUnloadPDAVO>> getBoxCabinetUnload4PDAPage(@RequestParam(value = "selfNo",required = false) String selfNo,
                                                                                       @RequestParam(value = "ulStatus",required = false) Integer ulStatus, PageVO page) {
        PageResult<BoxCabinetUnloadPDAVO> pageResult = boxCabinetUnloadService.getBoxCabinetUnload4PDAPage(selfNo,ulStatus, page);
        return success(pageResult);
    }
}
