package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxClearanceConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxClearanceDO;
import cn.iocoder.yudao.module.shipment.service.boxClearance.BoxClearanceService;
import cn.iocoder.yudao.module.shipment.vo.boxClearance.BoxClearanceBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxClearance.BoxClearanceCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxClearance.BoxClearanceQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxClearance.BoxClearanceUpdateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxClearanceOrder.BoxClearanceOrderCreateVO;
import cn.iocoder.yudao.module.shipment.vo.boxClearanceOrder.BoxClearanceOrderAllCreateVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 清关")
@RequestMapping("/ecw/box-clearance")
public class BoxClearanceController {

    @Resource
    private BoxClearanceService boxClearanceService;

    @PostMapping("/create")
    @ApiOperation("创建清关, use this")
    @Idempotent(timeout = 5)
//    @PreAuthorize("@ss.hasPermission('ecw:box-clearance:create')")
    public CommonResult<Long> createBoxClearance(@Valid @RequestBody BoxClearanceCreateReqVO createReqVO) {
        return success(boxClearanceService.createBoxClearance(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新清关, use this")
    @Idempotent(timeout = 5)
//    @PreAuthorize("@ss.hasPermission('ecw:box-clearance:update')")
    public CommonResult<Boolean> updateBoxClearance(@Valid @RequestBody BoxClearanceUpdateReqVO updateReqVO) {
        boxClearanceService.updateBoxClearance(updateReqVO);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得清关")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-clearance:query')")
    public CommonResult<BoxClearanceBackVO> getBoxClearance(@RequestParam("id") Long id) {
        BoxClearanceDO boxClearance = boxClearanceService.getBoxClearance(id);
        return success(BoxClearanceConvert.INSTANCE.convert(boxClearance));
    }

    @PostMapping("/updateOrderClearance")
    @ApiOperation("更新订单清关状态， use this")
    public CommonResult<Boolean> updateOrderClearance(@Valid @RequestBody BoxClearanceOrderCreateVO createReqVO) {
        boxClearanceService.updateOrderClearance(createReqVO);
        return success(true);
    }

    @PostMapping("/updateAllOrderClearance")
    @ApiOperation("更新所有订单到港状态， use this")
    public CommonResult<Boolean> updateAllOrderClearance(@Valid @RequestBody BoxClearanceOrderAllCreateVO createReqVO) {
        boxClearanceService.updateAllOrderClearance(createReqVO);
        return success(true);
    }

    @GetMapping("/list")
    @ApiOperation("获得清关列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-clearance:query')")
    public CommonResult<List<BoxClearanceBackVO>> getBoxClearanceList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxClearanceDO> list = boxClearanceService.getBoxClearanceList(ids);
        return success(BoxClearanceConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得清关分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-clearance:query')")
    public CommonResult<PageResult<BoxClearanceBackVO>> getBoxClearancePage(@Valid BoxClearanceQueryVO query, PageVO page) {
        PageResult<BoxClearanceDO> pageResult = boxClearanceService.getBoxClearancePage(query, page);
        return success(BoxClearanceConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出清关 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-clearance:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxClearanceExcel(@Valid BoxClearanceQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxClearanceDO> list = boxClearanceService.getBoxClearanceList(query);
        // 导出 Excel
        List<BoxClearanceBackVO> datas = BoxClearanceConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "清关.xls", "数据", BoxClearanceBackVO.class, datas);
    }
}
