package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxClearanceDocConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxClearanceDocDO;
import cn.iocoder.yudao.module.shipment.service.boxClearanceDoc.BoxClearanceDocService;
import cn.iocoder.yudao.module.shipment.vo.boxClearanceDoc.BoxClearanceDocBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxClearanceDoc.BoxClearanceDocCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxClearanceDoc.BoxClearanceDocQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxClearanceDoc.BoxClearanceDocUpdateReqVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 清关文件")
@RequestMapping("/ecw/box-clearance-doc")
public class BoxClearanceDocController {

    @Resource
    private BoxClearanceDocService boxClearanceDocService;

    @PostMapping("/create")
    @ApiOperation("创建清关文件, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-clearance-doc:create')")
    public CommonResult<Long> createBoxClearanceDoc(@Valid @RequestBody BoxClearanceDocCreateReqVO createReqVO) {
        return success(boxClearanceDocService.createBoxClearanceDoc(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新清关文件, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-clearance-doc:update')")
    public CommonResult<Boolean> updateBoxClearanceDoc(@Valid @RequestBody BoxClearanceDocUpdateReqVO updateReqVO) {
        boxClearanceDocService.updateBoxClearanceDoc(updateReqVO);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得清关文件")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-clearance-doc:query')")
    public CommonResult<BoxClearanceDocBackVO> getBoxClearanceDoc(@RequestParam("id") Long id) {
        BoxClearanceDocDO boxClearanceDoc = boxClearanceDocService.getBoxClearanceDoc(id);
        return success(BoxClearanceDocConvert.INSTANCE.convert(boxClearanceDoc));
    }

    @GetMapping("/list")
    @ApiOperation("获得清关文件列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-clearance-doc:query')")
    public CommonResult<List<BoxClearanceDocBackVO>> getBoxClearanceDocList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxClearanceDocDO> list = boxClearanceDocService.getBoxClearanceDocList(ids);
        return success(BoxClearanceDocConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得清关文件分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-clearance-doc:query')")
    public CommonResult<PageResult<BoxClearanceDocBackVO>> getBoxClearanceDocPage(@Valid BoxClearanceDocQueryVO query, PageVO page) {
        PageResult<BoxClearanceDocDO> pageResult = boxClearanceDocService.getBoxClearanceDocPage(query, page);
        return success(BoxClearanceDocConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/getDocZip")
    @ApiOperation("获得清关文件压缩包")
    @ApiImplicitParam(name = "shipmentId", value = "出货编号", required = true, example = "1024", dataTypeClass = Long.class)
    public void getDocZip(@RequestParam("shipmentId") Long shipmentId,HttpServletResponse response) {
        try {
            boxClearanceDocService.getDocZip(shipmentId,response.getOutputStream());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出清关文件 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-clearance-doc:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxClearanceDocExcel(@Valid BoxClearanceDocQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxClearanceDocDO> list = boxClearanceDocService.getBoxClearanceDocList(query);
        // 导出 Excel
        List<BoxClearanceDocBackVO> datas = BoxClearanceDocConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "清关文件.xls", "数据", BoxClearanceDocBackVO.class, datas);
    }
}
