package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.module.shipment.convert.BoxCustomsConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxCustomsDO;
import cn.iocoder.yudao.module.shipment.service.boxCustoms.BoxCustomsService;
import cn.iocoder.yudao.module.shipment.vo.boxCustoms.BoxCustomsBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxCustoms.BoxCustomsCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxCustoms.BoxCustomsQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxCustoms.BoxCustomsUpdateReqVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.text.MessageFormat;
import java.util.*;
import java.io.IOException;
import java.util.concurrent.TimeUnit;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import static cn.iocoder.yudao.module.shipment.controller.admin.constant.Constant.BOX_UPDATE_KEY;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_UPDATE_REPEAT_COMMIT;

@Validated
@RestController
@Api(tags = "管理后台 - 报关")
@RequestMapping("/ecw/box-customs")
public class BoxCustomsController {

    @Resource
    private BoxCustomsService boxCustomsService;

    @Resource
    private RedisHelper redisHelper;

    @PostMapping("/create")
    @ApiOperation("创建报关，use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-customs:create')")
    public CommonResult<Long> createBoxCustoms(@Valid @RequestBody BoxCustomsCreateReqVO createReqVO) {
        return success(boxCustomsService.createBoxCustoms(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新报关, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-customs:update')")
    public CommonResult<Boolean> updateBoxCustoms(@Valid @RequestBody BoxCustomsUpdateReqVO updateReqVO) {
        String redisKey = MessageFormat.format(BOX_UPDATE_KEY, updateReqVO.getShipmentId().toString());
        Long count = redisHelper.incrBy(redisKey, 1);
        if (count > 1) {
            return error(BOX_UPDATE_REPEAT_COMMIT);
        }
        redisHelper.expire(redisKey, 1, TimeUnit.MINUTES);
        try {
            boxCustomsService.updateBoxCustoms(updateReqVO);
        }catch (Exception e){
            throw e;
        } finally {
            redisHelper.delete(redisKey);
        }
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得报关")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-customs:query')")
    public CommonResult<BoxCustomsBackVO> getBoxCustoms(@RequestParam("id") Long id) {
        BoxCustomsDO boxCustoms = boxCustomsService.getBoxCustoms(id);
        return success(BoxCustomsConvert.INSTANCE.convert(boxCustoms));
    }

    @GetMapping("/list")
    @ApiOperation("获得报关列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-customs:query')")
    public CommonResult<List<BoxCustomsBackVO>> getBoxCustomsList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxCustomsDO> list = boxCustomsService.getBoxCustomsList(ids);
        return success(BoxCustomsConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得报关分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-customs:query')")
    public CommonResult<PageResult<BoxCustomsBackVO>> getBoxCustomsPage(@Valid BoxCustomsQueryVO query, PageVO page) {
        PageResult<BoxCustomsDO> pageResult = boxCustomsService.getBoxCustomsPage(query, page);
        return success(BoxCustomsConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出报关 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-customs:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxCustomsExcel(@Valid BoxCustomsQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxCustomsDO> list = boxCustomsService.getBoxCustomsList(query);
        // 导出 Excel
        List<BoxCustomsBackVO> datas = BoxCustomsConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "报关.xls", "数据", BoxCustomsBackVO.class, datas);
    }
}
