package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxCustomsExtraCostConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxCustomsExtraCostDO;
import cn.iocoder.yudao.module.shipment.service.boxCustomsExtraCost.BoxCustomsExtraCostService;
import cn.iocoder.yudao.module.shipment.vo.boxCustomsExtraCost.BoxCustomsExtraCostBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxCustomsExtraCost.BoxCustomsExtraCostCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxCustomsExtraCost.BoxCustomsExtraCostQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxCustomsExtraCost.BoxCustomsExtraCostUpdateReqVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 额外报关费用")
@RequestMapping("/ecw/box-customs-extra-cost")
public class BoxCustomsExtraCostController {

    @Resource
    private BoxCustomsExtraCostService boxCustomsExtraCostService;

    @PostMapping("/searchByOrderNo")
    @ApiOperation("搜索出货单下的订单，use this")
    public CommonResult<Long> searchByOrderNo(@Valid @RequestBody BoxCustomsExtraCostCreateReqVO createReqVO) {
        Long orderId = boxCustomsExtraCostService.searchByOrderNo(createReqVO);
        return success(orderId);
    }

    @PostMapping("/create")
    @ApiOperation("创建额外报关费用, use this")
    public CommonResult<Long> createBoxCustomsExtraCost(@Valid @RequestBody BoxCustomsExtraCostCreateReqVO createReqVO) {
        return success(boxCustomsExtraCostService.createBoxCustomsExtraCost(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新额外报关费用, use this")
    public CommonResult<Boolean> updateBoxCustomsExtraCost(@Valid @RequestBody BoxCustomsExtraCostUpdateReqVO updateReqVO) {
        boxCustomsExtraCostService.updateBoxCustomsExtraCost(updateReqVO);
        return success(true);
    }

    @GetMapping("/list")
    @ApiOperation("获得额外报关费用列表, use this")
    public CommonResult<List<BoxCustomsExtraCostBackVO>> getBoxCustomsExtraCostList(BoxCustomsExtraCostQueryVO query) {
        List<BoxCustomsExtraCostDO> list = boxCustomsExtraCostService.getBoxCustomsExtraCostList(query);
        return success(BoxCustomsExtraCostConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/resetById")
    @ApiOperation("单个费用重置，use this")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<Boolean> resetById(@RequestParam("id") Long id) {
        boxCustomsExtraCostService.resetById(id);
        return success(true);
    }

    @GetMapping("/resetByShipmentId")
    @ApiOperation("出货单费用重置, use this")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<Boolean> resetByShipmentId(@RequestParam("id") Long id) {
        boxCustomsExtraCostService.resetByShipmentId(id);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除额外报关费用")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-customs-extra-cost:delete')")
    public CommonResult<Boolean> deleteBoxCustomsExtraCost(@RequestParam("id") Long id) {
        boxCustomsExtraCostService.deleteBoxCustomsExtraCost(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得额外报关费用")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-customs-extra-cost:query')")
    public CommonResult<BoxCustomsExtraCostBackVO> getBoxCustomsExtraCost(@RequestParam("id") Long id) {
        BoxCustomsExtraCostDO boxCustomsExtraCost = boxCustomsExtraCostService.getBoxCustomsExtraCost(id);
        return success(BoxCustomsExtraCostConvert.INSTANCE.convert(boxCustomsExtraCost));
    }



    @GetMapping("/page")
    @ApiOperation("获得额外报关费用分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-customs-extra-cost:query')")
    public CommonResult<PageResult<BoxCustomsExtraCostBackVO>> getBoxCustomsExtraCostPage(@Valid BoxCustomsExtraCostQueryVO query, PageVO page) {
        PageResult<BoxCustomsExtraCostDO> pageResult = boxCustomsExtraCostService.getBoxCustomsExtraCostPage(query, page);
        return success(BoxCustomsExtraCostConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出额外报关费用 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-customs-extra-cost:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxCustomsExtraCostExcel(@Valid BoxCustomsExtraCostQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxCustomsExtraCostDO> list = boxCustomsExtraCostService.getBoxCustomsExtraCostList(query);
        // 导出 Excel
        List<BoxCustomsExtraCostBackVO> datas = BoxCustomsExtraCostConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "额外报关费用.xls", "数据", BoxCustomsExtraCostBackVO.class, datas);
    }
}
