package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.common.exception.ErrorCode;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxLadingCopyConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxLadingCopyDO;
import cn.iocoder.yudao.module.shipment.service.boxLadingCopy.BoxLadingCopyService;
import cn.iocoder.yudao.module.shipment.vo.boxLadingCopy.BoxLadingCopyBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxLadingCopy.BoxLadingCopyCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxLadingCopy.BoxLadingCopyQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxLadingCopy.BoxLadingCopyUpdateReqVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLDecoder;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.FILE_NOT_EXISTS;

@Validated
@RestController
@Api(tags = "管理后台 - 提单copy")
@RequestMapping("/ecw/box-lading-copy")
public class BoxLadingCopyController {

    @Resource
    private BoxLadingCopyService boxLadingCopyService;

    @PostMapping("/create")
    @ApiOperation("创建提单copy，use this")
//    //@PreAuthorize("@ss.hasPermission('ecw:box-lading-copy:create')")
    public CommonResult<Long> createBoxLadingCopy(@Valid @RequestBody BoxLadingCopyCreateReqVO createReqVO) {
        return success(boxLadingCopyService.createBoxLadingCopy(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新提单copy, use this")
    //@PreAuthorize("@ss.hasPermission('ecw:box-lading-copy:update')")
    public CommonResult<Boolean> updateBoxLadingCopy(@Valid @RequestBody BoxLadingCopyUpdateReqVO updateReqVO) {
        boxLadingCopyService.updateBoxLadingCopy(updateReqVO);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得提单copy")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-lading-copy:query')")
    public CommonResult<BoxLadingCopyBackVO> getBoxLadingCopy(@RequestParam("id") Long id) {
        BoxLadingCopyDO boxLadingCopy = boxLadingCopyService.getBoxLadingCopy(id);
        return success(BoxLadingCopyConvert.INSTANCE.convert(boxLadingCopy));
    }

    @GetMapping("/downloadLadingCopy")
    @ApiOperation("下载提单copy")
    @ApiImplicitParam(name = "shipmentId", value = "出货ID", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-lading-copy:query')")
    public CommonResult<String> downloadLadingCopy(@RequestParam("shipmentId") Long shipmentId) {
        BoxLadingCopyDO boxLadingCopy = boxLadingCopyService.selectOne(BoxLadingCopyDO::getShipmentId,shipmentId);
        if(boxLadingCopy !=null && StringUtils.isNotBlank(boxLadingCopy.getCopyUrl())){
            return success(boxLadingCopy.getCopyUrl());
        }
        return error(FILE_NOT_EXISTS);
    }

    @GetMapping("/list")
    @ApiOperation("获得提单copy列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-lading-copy:query')")
    public CommonResult<List<BoxLadingCopyBackVO>> getBoxLadingCopyList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxLadingCopyDO> list = boxLadingCopyService.getBoxLadingCopyList(ids);
        return success(BoxLadingCopyConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得提单copy分页")
    //@PreAuthorize("@ss.hasPermission('ecw:box-lading-copy:query')")
    public CommonResult<PageResult<BoxLadingCopyBackVO>> getBoxLadingCopyPage(@Valid BoxLadingCopyQueryVO query, PageVO page) {
        PageResult<BoxLadingCopyDO> pageResult = boxLadingCopyService.getBoxLadingCopyPage(query, page);
        return success(BoxLadingCopyConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出提单copy Excel")
    //@PreAuthorize("@ss.hasPermission('ecw:box-lading-copy:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxLadingCopyExcel(@Valid BoxLadingCopyQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxLadingCopyDO> list = boxLadingCopyService.getBoxLadingCopyList(query);
        // 导出 Excel
        List<BoxLadingCopyBackVO> datas = BoxLadingCopyConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "提单copy.xls", "数据", BoxLadingCopyBackVO.class, datas);
    }
}
