package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxLoadInfoConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxLoadInfoDO;
import cn.iocoder.yudao.module.shipment.service.boxLoadInfo.BoxLoadInfoService;
import cn.iocoder.yudao.module.shipment.vo.boxLoadInfo.*;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 已装柜标签")
@RequestMapping("/ecw/box-load-info")
public class BoxLoadInfoController {

    @Resource
    private BoxLoadInfoService boxLoadInfoService;

    @PostMapping("/singleCreate")
    @ApiOperation("创建已装柜标签, use this")
    @Idempotent(timeout = 5)
    public CommonResult<Long> singleCreate(@Valid @RequestBody BoxLoadInfoCreateReqVO createReqVO) {
        return success(boxLoadInfoService.singleCreate(createReqVO));
    }

    @PostMapping("/singleAirCreate")
    @ApiOperation("创建出仓标签, use this")
    @Idempotent(timeout = 5)
    public CommonResult<Long> singleAirCreate(@RequestBody BoxLoadInfoCreateReqVO createReqVO) {
        return success(boxLoadInfoService.singleAirCreate(createReqVO));
    }

    @PostMapping("/batchCreate")
    @ApiOperation("批量装柜, use this")
    @Idempotent(timeout = 5)
    public CommonResult<Long> batchCreate(@Valid @RequestBody BoxLoadInfoCreateReqVO createReqVO) {
        return success(boxLoadInfoService.batchCreate(createReqVO));
    }

    @PostMapping("/batchAirCreate")
    @ApiOperation("批量出仓, use this")
    @Idempotent(timeout = 5)
    public CommonResult<Long> batchAirCreate(@RequestBody BoxLoadInfoCreateReqVO createReqVO) {
        return success(boxLoadInfoService.batchAirCreate(createReqVO));
    }

    @PostMapping("/externalLoad")
    @ApiOperation("外部仓装柜, use this")
    @Idempotent(timeout = 5)
    public CommonResult<Long> externalLoad(@Valid @RequestBody BoxExternalLoadReq createReqVO) {
        return success(boxLoadInfoService.externalLoad(createReqVO));
    }

    @PostMapping("/singleDelete")
    @ApiOperation("删除已装柜标签, use this")
    public CommonResult<Boolean> singleDelete(@RequestBody BoxLoadInfoCreateReqVO createReqVO) {
        boxLoadInfoService.deleteBoxLoadInfo(createReqVO);
        return success(true);
    }

    @PostMapping("/batchDelete")
    @ApiOperation("批量删除已装柜标签, use this")
    public CommonResult<Boolean> batchDelete(@RequestBody BoxLoadInfoCreateReqVO createReqVO) {
        boxLoadInfoService.deleteBoxLoadInfo(createReqVO);
        return success(true);
    }

    @PostMapping("/singleUnload")
    @ApiOperation("单个卸柜标签, use this")
    public CommonResult<Boolean> singleUnload(@Valid @RequestBody BoxLoadInfoCreateReqVO createReqVO) {
        boxLoadInfoService.unloadBoxInfo(createReqVO);
        return success(true);
    }

    @PostMapping("/batchUnload")
    @ApiOperation("批量卸柜标签, use this")
    public CommonResult<Boolean> batchUnload(@Valid @RequestBody BoxLoadInfoCreateReqVO createReqVO) {
        boxLoadInfoService.unloadBoxInfo(createReqVO);
        return success(true);
    }

    @PostMapping("/allUnload")
    @ApiOperation("卸柜所有标签, use this")
    public CommonResult<Boolean> allUnload(@Valid @RequestBody BoxLoadInfoCreateReqVO createReqVO) {
        boxLoadInfoService.unloadAllBoxInfo(createReqVO);
        return success(true);
    }

    @PostMapping("/airSingleUnload")
    @ApiOperation("单个卸柜标签, use this")
    public CommonResult<Boolean> airSingleUnload(@Valid @RequestBody BoxLoadInfoCreateReqVO createReqVO) {
        boxLoadInfoService.airUnloadBoxInfo(createReqVO);
        return success(true);
    }

    @PostMapping("/airBatchUnload")
    @ApiOperation("批量卸柜标签, use this")
    public CommonResult<Boolean> airBatchUnload(@Valid @RequestBody BoxLoadInfoCreateReqVO createReqVO) {
        boxLoadInfoService.airUnloadBoxInfo(createReqVO);
        return success(true);
    }

    @PostMapping("/airAllUnload")
    @ApiOperation("空运卸柜所有标签, use this")
    public CommonResult<Boolean> airAllUnload(@Valid @RequestBody BoxLoadInfoCreateReqVO createReqVO) {
        boxLoadInfoService.unloadAllAirBoxInfo(createReqVO);
        return success(true);
    }

    @PostMapping("/remove")
    @ApiOperation("装柜移出, use this")
    public CommonResult<Boolean> removeBox(@RequestBody BoxLoadInfoRemoveReqVO removeReqVO) {
        boxLoadInfoService.removeBox(removeReqVO);
        return success(true);
    }

    @PostMapping("/orderTagList")
    @ApiOperation("订单装箱编号列表, use this")
    @ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<BoxLoadInfoOrderVO> orderTagList(@RequestParam("orderId") Long orderId) {
        return boxLoadInfoService.orderTagList(orderId);
    }

    @PostMapping("/boxUpdate")
    @ApiOperation("修改柜信息, use this")
    public CommonResult<Boolean> boxUpdate(@RequestBody BoxInfoUpdateReq boxInfoUpdateReq) {
        boxLoadInfoService.boxUpdate(boxInfoUpdateReq);
        return success(true);
    }


    @PutMapping("/update")
    @ApiOperation("更新已装柜标签")
    @PreAuthorize("@ss.hasPermission('ecw:box-load-info:update')")
    public CommonResult<Boolean> updateBoxLoadInfo(@Valid @RequestBody BoxLoadInfoUpdateReqVO updateReqVO) {
        boxLoadInfoService.updateBoxLoadInfo(updateReqVO);
        return success(true);
    }



    @GetMapping("/get")
    @ApiOperation("获得已装柜标签")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-load-info:query')")
    public CommonResult<BoxLoadInfoBackVO> getBoxLoadInfo(@RequestParam("id") Long id) {
        BoxLoadInfoDO boxLoadInfo = boxLoadInfoService.getBoxLoadInfo(id);
        return success(BoxLoadInfoConvert.INSTANCE.convert(boxLoadInfo));
    }

    @GetMapping("/list")
    @ApiOperation("获得已装柜标签列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-load-info:query')")
    public CommonResult<List<BoxLoadInfoBackVO>> getBoxLoadInfoList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxLoadInfoDO> list = boxLoadInfoService.getBoxLoadInfoList(ids);
        return success(BoxLoadInfoConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得已装柜标签分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-load-info:query')")
    public CommonResult<PageResult<BoxLoadInfoBackVO>> getBoxLoadInfoPage(@Valid BoxLoadInfoQueryVO query, PageVO page) {
        PageResult<BoxLoadInfoDO> pageResult = boxLoadInfoService.getBoxLoadInfoPage(query, page);
        return success(BoxLoadInfoConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出已装柜标签 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-load-info:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxLoadInfoExcel(@Valid BoxLoadInfoQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxLoadInfoDO> list = boxLoadInfoService.getBoxLoadInfoList(query);
        // 导出 Excel
        List<BoxLoadInfoBackVO> datas = BoxLoadInfoConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "已装柜标签.xls", "数据", BoxLoadInfoBackVO.class, datas);
    }
}
