package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.shipment.convert.BoxMergePkgConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxMergePkgDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxPkgOrderDO;
import cn.iocoder.yudao.module.shipment.dto.BoxMergePkgPageDto;
import cn.iocoder.yudao.module.shipment.service.boxMergePkg.BoxMergePkgService;
import cn.iocoder.yudao.module.shipment.service.boxPkgOrder.BoxPkgOrderService;
import cn.iocoder.yudao.module.shipment.vo.boxMergePkg.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.context.annotation.Lazy;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

@Validated
@RestController
@Api(tags = "管理后台 - 合包箱")
@RequestMapping("/ecw/box-merge-pkg")
public class BoxMergePkgController {

    @Resource
    private BoxMergePkgService boxMergePkgService;

    @Resource
    @Lazy
    private BoxPkgOrderService boxPkgOrderService;

    @PostMapping("/create")
    @ApiOperation("创建合包箱")
    //@PreAuthorize("@ss.hasPermission('ecw:box-merge-pkg:create')")
    public CommonResult<String> createBoxMergePkg(
            @Valid @RequestBody BoxMergePkgCreateReqVO createReqVO) {
        return success(boxMergePkgService.createBoxMergePkg(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新合包箱")
    //@PreAuthorize("@ss.hasPermission('ecw:box-merge-pkg:update')")
    public CommonResult<Boolean> updateBoxMergePkg(
            @Valid @RequestBody BoxMergePkgUpdateReqVO updateReqVO) {
        boxMergePkgService.updateBoxMergePkg(updateReqVO);
        return success(true);
    }

    @PutMapping("/closeMergePkg")
    @ApiOperation("封箱")
    //@PreAuthorize("@ss.hasPermission('ecw:box-merge-pkg:update')")
    public CommonResult<Boolean> closeMergePkg(@Valid @RequestBody BoxMergePkgUpdateReqVO updateReqVO) {
        boxMergePkgService.closeMergePkg(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除合包箱")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-merge-pkg:delete')")
    public CommonResult<Boolean> deleteBoxMergePkg(@RequestParam("id") Long id) {
        boxMergePkgService.deleteBoxMergePkg(id);
        boxPkgOrderService.remove(new LambdaQueryWrapperX<BoxPkgOrderDO>().eq(BoxPkgOrderDO::getPkgId,id));
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得合包箱")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-merge-pkg:query')")
    public CommonResult<BoxMergePkgBackVO> getBoxMergePkg(@RequestParam("id") Long id) {
        BoxMergePkgDO boxMergePkg = boxMergePkgService.getBoxMergePkg(id);
        return success(BoxMergePkgConvert.INSTANCE.convert(boxMergePkg));
    }

    @GetMapping("/list")
    @ApiOperation("获得合包箱列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-merge-pkg:query')")
    public CommonResult<List<BoxMergePkgBackVO>> getBoxMergePkgList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxMergePkgDO> list = boxMergePkgService.getBoxMergePkgList(ids);
        return success(BoxMergePkgConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得合包箱分页")
    //@PreAuthorize("@ss.hasPermission('ecw:box-merge-pkg:query')")
    public CommonResult<PageResult<BoxMergePkgPageDto>> getBoxMergePkgPage(@Valid BoxMergePkgQueryVO query, PageVO page) {
        PageResult<BoxMergePkgPageDto> pageResult =
                boxMergePkgService.getBoxMergePkgPage(query, page);
        return success(pageResult);
    }

    @GetMapping("/getUnPkgPage")
    @ApiOperation("获得未装箱列表分页")
//    //@PreAuthorize("@ss.hasPermission('ecw:box-merge-pkg:query')")
    public CommonResult<PageResult<BoxMergePkgWebVO>> getUnPkgPage(PageVO page,@RequestParam("shipmentId") Long shipmentId) {
        PageResult<BoxMergePkgWebVO> pageResult =
                boxMergePkgService.getUnPkgPage(page,shipmentId);
        return success(pageResult);
    }

    @GetMapping("/getPkgPageByPkgId")
    @ApiOperation("获得已装箱列表分页")
//    //@PreAuthorize("@ss.hasPermission('ecw:box-merge-pkg:query')")
    public CommonResult<PageResult<BoxMergePkgWebVO>> getPkgPageByPkgId(BoxMergePkgUpdateReqVO boxMergePkgUpdateReqVO,PageVO page) {
        PageResult<BoxMergePkgWebVO> pageResult = boxMergePkgService.getPkgPageByPkgId(boxMergePkgUpdateReqVO,page);
        return success(pageResult);
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出合包箱 Excel")
    //@PreAuthorize("@ss.hasPermission('ecw:box-merge-pkg:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxMergePkgExcel(@Valid BoxMergePkgQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxMergePkgDO> list = boxMergePkgService.getBoxMergePkgList(query);
        // 导出 Excel
        List<BoxMergePkgBackVO> datas = BoxMergePkgConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "合包箱.xls", "数据", BoxMergePkgBackVO.class, datas);
    }

    @GetMapping("/getMergePkgInfoById")
    @ApiOperation("获得封箱数据")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-pkg-order:query')")
    public CommonResult<BoxMergePkgBackVO> getMergePkgInfoById(
            @RequestParam("id") Long id) {
        //
        BoxMergePkgDO boxMergePkgBackVO = boxMergePkgService.getMergePkgInfoById(id);
        return success(BoxMergePkgConvert.INSTANCE.convert(boxMergePkgBackVO));
    }

    @GetMapping("/getMergeTagById")
    @ApiOperation("获得封箱标签数据")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-pkg-order:query')")
    public CommonResult<BoxMergePkgTagVO> getMergeTagById(@RequestParam("id") Long id) {
        BoxMergePkgTagVO mergePkgTagVO = boxMergePkgService.getMergeTagById(id);
        return success(mergePkgTagVO);
    }

    @GetMapping("/finishMergePkg")
    @ApiOperation("完成合包")
    @ApiImplicitParam(name = "shipmentId", value = "出货编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-pkg-order:query')")
    public CommonResult<String> finishMergePkg(@RequestParam("shipmentId") Long shipmentId) {
        boxMergePkgService.finishMergePkg(shipmentId);
        return success("success");
    }

    @GetMapping("/getMergePkgByPkgNum")
    @ApiOperation("PDA - 根据合包箱号获取合包下订单数据")
    @ApiImplicitParam(name = "pkgNum", value = "合包箱号", required = true, example = "M10001", dataTypeClass = String.class)
    public CommonResult<List<BoxMergePkgPDAVO>> getMergePkgByPkgNum(@RequestParam("pkgNum") String pkgNum) {
        List<BoxMergePkgPDAVO> boxMergePkgPDAVOS = new ArrayList<>();
        try {
            boxMergePkgPDAVOS = boxMergePkgService.getMergePkgByPkgNum(pkgNum);
        } catch (ServiceException e) {
            return error(e.getCode(),e.getMessage());
        }
        return success(boxMergePkgPDAVOS);
    }

    @GetMapping("/getBoxMergePkgList4PDAPage")
    @ApiOperation("PDA - 获得空运合包装箱分页")
    public CommonResult<PageResult<BoxMergePkgPDAListVO>> getBoxMergePkgList4PDAPage(@RequestParam(value = "selfNo",required = false) String selfNo, PageVO page) {
        PageResult<BoxMergePkgPDAListVO> pageResult = boxMergePkgService.getBoxMergePkgList4PDAPage(selfNo, page);
        return success(pageResult);
    }
}
