package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxOpLogConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxOpLogDO;
import cn.iocoder.yudao.module.shipment.service.boxOpLog.BoxOpLogService;
import cn.iocoder.yudao.module.shipment.vo.boxOpLog.BoxOpLogBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxOpLog.BoxOpLogQueryVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 出货操作日志")
@RequestMapping("/ecw/box-op-log")
public class BoxOpLogController {

    @Resource
    private BoxOpLogService boxOpLogService;

    @GetMapping("/get")
    @ApiOperation("获得出货操作日志")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-op-log:query')")
    public CommonResult<BoxOpLogBackVO> getBoxOpLog(@RequestParam("id") Long id) {
        BoxOpLogDO boxOpLog = boxOpLogService.getBoxOpLog(id);
        return success(BoxOpLogConvert.INSTANCE.convert(boxOpLog));
    }

    /*
     获取订单的操作日志
     */
    @GetMapping("/list")
    @ApiOperation("获得出货操作日志列表， use this")
    //@PreAuthorize("@ss.hasPermission('ecw:box-op-log:query')")
    public CommonResult<List<BoxOpLogBackVO>> getBoxOpLogList(BoxOpLogQueryVO query) {
        List<BoxOpLogDO> list = boxOpLogService.getBoxOpLogList(query);
        List<BoxOpLogBackVO> resultList = BoxOpLogConvert.INSTANCE.convertList(list);
        boxOpLogService.matchBpmProcessId(resultList);
        return success(resultList);
    }

    @GetMapping("/page")
    @ApiOperation("获得出货操作日志分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-op-log:query')")
    public CommonResult<PageResult<BoxOpLogBackVO>> getBoxOpLogPage(@Valid BoxOpLogQueryVO query, PageVO page) {
        PageResult<BoxOpLogDO> pageResult = boxOpLogService.getBoxOpLogPage(query, page);
        return success(BoxOpLogConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出出货操作日志 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-op-log:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxOpLogExcel(@Valid BoxOpLogQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxOpLogDO> list = boxOpLogService.getBoxOpLogList(query);
        // 导出 Excel
        List<BoxOpLogBackVO> datas = BoxOpLogConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "出货操作日志.xls", "数据", BoxOpLogBackVO.class, datas);
    }
}
