package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.shipment.convert.boxOrderCheckInfo.BoxOrderCheckInfoConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.boxOrderCheckInfo.BoxOrderCheckInfoDO;
import cn.iocoder.yudao.module.shipment.dto.OrderCheckInfoDto;
import cn.iocoder.yudao.module.shipment.service.boxOrderCheckInfo.BoxOrderCheckInfoService;
import cn.iocoder.yudao.module.shipment.vo.box.BoxOrderSearchVO;
import cn.iocoder.yudao.module.shipment.vo.boxLoadInfo.BoxLoadInfoOrderVO;
import cn.iocoder.yudao.module.shipment.vo.boxOrderCheckInfo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

@Validated
@RestController
@Api(tags = "管理后台 - 理货和合包标签")
@RequestMapping("/ecw/box-order-check-info")
public class BoxOrderCheckInfoController {

    @Resource
    private BoxOrderCheckInfoService boxOrderCheckInfoService;

    @PostMapping("/singleCreate")
    @ApiOperation("创建理货和合包标签")
    @Idempotent(timeout = 5)
    //@PreAuthorize("@ss.hasPermission('ecw:box-order-check-info:create')")
    public CommonResult<Long> createBoxOrderCheckInfo(@Valid @RequestBody BoxOrderCheckInfoCreateReqVO createReqVO) {
        return success(boxOrderCheckInfoService.createBoxOrderCheckInfo(createReqVO));
    }

    @PostMapping("/batchCreate")
    @ApiOperation("批量创建理货和合包标签, use this")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> batchCreate(@Valid @RequestBody BoxOrderCheckInfoCreateReqVO createReqVO) {
        return success(boxOrderCheckInfoService.batchCreate(createReqVO));
    }

    @PostMapping("/singleDelete")
    @ApiOperation("删除已创建理货和合包标签, use this")
    public CommonResult<Boolean> singleDelete(@RequestBody BoxOrderCheckInfoCreateReqVO createReqVO) {
        boxOrderCheckInfoService.deleteBoxLoadInfo(createReqVO);
        return success(true);
    }

    @PostMapping("/batchDelete")
    @ApiOperation("批量删除已创建理货和合包标签, use this")
    public CommonResult<Boolean> batchDelete(@RequestBody BoxOrderCheckInfoCreateReqVO createReqVO) {
        boxOrderCheckInfoService.deleteBoxLoadInfo(createReqVO);
        return success(true);
    }

    @GetMapping("/orderTagList")
    @ApiOperation("订单装箱编号列表, use this")
    @ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<BoxLoadInfoOrderVO> orderTagList(@RequestParam("orderId") Long orderId,@RequestParam("type") Integer type) {
        return boxOrderCheckInfoService.orderTagList(orderId,type);
    }

    @PostMapping("/batchRemove")
    @ApiOperation("批量纠错")
    public CommonResult<Boolean> batchRemove(@Valid @RequestBody BoxOrderCheckInfoRemoveReqVO removeReqVO) {
        boxOrderCheckInfoService.batchRemove(removeReqVO);
        return success(true);
    }

    @PutMapping("/update")
    @ApiOperation("更新理货和合包标签")
    //@PreAuthorize("@ss.hasPermission('ecw:box-order-check-info:update')")
    public CommonResult<Boolean> updateBoxOrderCheckInfo(@Valid @RequestBody BoxOrderCheckInfoUpdateReqVO updateReqVO) {
        boxOrderCheckInfoService.updateBoxOrderCheckInfo(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除理货和合包标签")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-order-check-info:delete')")
    public CommonResult<Boolean> deleteBoxOrderCheckInfo(@RequestParam("id") Long id) {
        boxOrderCheckInfoService.deleteBoxOrderCheckInfo(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得理货和合包标签")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-order-check-info:query')")
    public CommonResult<BoxOrderCheckInfoBackVO> getBoxOrderCheckInfo(@RequestParam("id") Long id) {
        BoxOrderCheckInfoDO boxOrderCheckInfo = boxOrderCheckInfoService.getBoxOrderCheckInfo(id);
        return success(BoxOrderCheckInfoConvert.INSTANCE.convert(boxOrderCheckInfo));
    }

    @GetMapping("/list")
    @ApiOperation("获得理货和合包标签列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('ecw:box-order-check-info:query')")
    public CommonResult<List<BoxOrderCheckInfoBackVO>> getBoxOrderCheckInfoList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxOrderCheckInfoDO> list = boxOrderCheckInfoService.getBoxOrderCheckInfoList(ids);
        return success(BoxOrderCheckInfoConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得理货和合包标签分页")
    //@PreAuthorize("@ss.hasPermission('ecw:box-order-check-info:query')")
    public CommonResult<PageResult<BoxOrderCheckInfoBackVO>> getBoxOrderCheckInfoPage(@Valid BoxOrderCheckInfoQueryVO query, PageVO page) {
        PageResult<BoxOrderCheckInfoDO> pageResult = boxOrderCheckInfoService.getBoxOrderCheckInfoPage(query, page);
        return success(BoxOrderCheckInfoConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出理货和合包标签 Excel")
    //@PreAuthorize("@ss.hasPermission('ecw:box-order-check-info:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxOrderCheckInfoExcel(@Valid BoxOrderCheckInfoQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxOrderCheckInfoDO> list = boxOrderCheckInfoService.getBoxOrderCheckInfoList(query);
        // 导出 Excel
        List<BoxOrderCheckInfoBackVO> datas = BoxOrderCheckInfoConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "理货和合包标签.xls", "数据", BoxOrderCheckInfoBackVO.class, datas);
    }

    @PostMapping("/searchLoadOrderByBoxNo")
    @ApiOperation("根据箱号、订单号查找订单号码，理货和合包使用")
    public CommonResult<OrderCheckInfoDto> searchLoadOrderByBoxNo(@RequestBody BoxOrderSearchVO boxOrderSearchVO) {
        return success(boxOrderCheckInfoService.searchLoadOrderByBoxNo(boxOrderSearchVO));
    }
}
