package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.order.vo.order.OrderPreloadReq;
import cn.iocoder.yudao.module.shipment.convert.BoxPreloadSectionConvert;
import cn.iocoder.yudao.module.shipment.convert.BoxSettlementConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxPreloadSectionDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxSettlementDO;
import cn.iocoder.yudao.module.shipment.service.boxPreloadSection.BoxPreloadSectionService;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.PreloadedReq;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadSection.BoxPreloadSectionBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadSection.BoxPreloadSectionCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadSection.BoxPreloadSectionQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxSettlement.BoxSettlementBackVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 预装部分")
@RequestMapping("/ecw/box-preload-section")
public class BoxPreloadSectionController {

    @Resource
    private BoxPreloadSectionService boxPreloadSectionService;

    @PostMapping("/create")
    @ApiOperation("创建预装部分, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-preload-section:create')")
    public CommonResult<Long> createBoxPreloadSection(@Valid @RequestBody BoxPreloadSectionCreateReqVO createReqVO) {
        return success(boxPreloadSectionService.createBoxPreloadSection(createReqVO));
    }


    @GetMapping("/list")
    @ApiOperation("获得预装列表, use this,查询出货单下的所有部分列表")
    public CommonResult<List<BoxPreloadSectionBackVO>> getBoxPreloadSectionList(@Valid BoxPreloadSectionQueryVO query) {
        List<BoxPreloadSectionDO> list = boxPreloadSectionService.getBoxPreloadSectionList(query);
        list.sort(Comparator.comparing(BoxPreloadSectionDO::getId));
        return success(BoxPreloadSectionConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/get")
    @ApiOperation("获得预装部分")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-preload-section:query')")
    public CommonResult<BoxPreloadSectionBackVO> getBoxPreloadSection(@RequestParam("id") Long id) {
        BoxPreloadSectionDO boxPreloadSection = boxPreloadSectionService.getBoxPreloadSection(id);
        return success(BoxPreloadSectionConvert.INSTANCE.convert(boxPreloadSection));
    }

    @GetMapping("/page")
    @ApiOperation("获得预装部分分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-preload-section:query')")
    public CommonResult<PageResult<BoxPreloadSectionBackVO>> getBoxPreloadSectionPage(@Valid BoxPreloadSectionQueryVO query, PageVO page) {
        PageResult<BoxPreloadSectionDO> pageResult = boxPreloadSectionService.getBoxPreloadSectionPage(query, page);
        return success(BoxPreloadSectionConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出预装部分 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-preload-section:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxPreloadSectionExcel(@Valid BoxPreloadSectionQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxPreloadSectionDO> list = boxPreloadSectionService.getBoxPreloadSectionList(query);
        // 导出 Excel
        List<BoxPreloadSectionBackVO> datas = BoxPreloadSectionConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "预装部分.xls", "数据", BoxPreloadSectionBackVO.class, datas);
    }
}
