package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxSettlementConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxSettlementDO;
import cn.iocoder.yudao.module.shipment.service.boxSettlement.BoxSettlementService;
import cn.iocoder.yudao.module.shipment.vo.boxSettlement.BoxSettlementBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxSettlement.BoxSettlementCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxSettlement.BoxSettlementQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxSettlement.BoxSettlementUpdateReqVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 结算")
@RequestMapping("/ecw/box-settlement")
public class BoxSettlementController {

    @Resource
    private BoxSettlementService boxSettlementService;

    @PostMapping("/create")
    @ApiOperation("创建结算, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-settlement:create')")
    public CommonResult<Long> createBoxSettlement(@Valid @RequestBody BoxSettlementCreateReqVO createReqVO) {
        return success(boxSettlementService.createBoxSettlement(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新结算, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-settlement:update')")
    public CommonResult<Boolean> updateBoxSettlement(@Valid @RequestBody BoxSettlementUpdateReqVO updateReqVO) {
        boxSettlementService.updateBoxSettlement(updateReqVO);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得结算")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-settlement:query')")
    public CommonResult<BoxSettlementBackVO> getBoxSettlement(@RequestParam("id") Long id) {
        BoxSettlementDO boxSettlement = boxSettlementService.getBoxSettlement(id);
        return success(BoxSettlementConvert.INSTANCE.convert(boxSettlement));
    }

    @GetMapping("/list")
    @ApiOperation("获得结算列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-settlement:query')")
    public CommonResult<List<BoxSettlementBackVO>> getBoxSettlementList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxSettlementDO> list = boxSettlementService.getBoxSettlementList(ids);
        return success(BoxSettlementConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得结算分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-settlement:query')")
    public CommonResult<PageResult<BoxSettlementBackVO>> getBoxSettlementPage(@Valid BoxSettlementQueryVO query, PageVO page) {
        PageResult<BoxSettlementDO> pageResult = boxSettlementService.getBoxSettlementPage(query, page);
        return success(BoxSettlementConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出结算 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-settlement:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxSettlementExcel(@Valid BoxSettlementQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxSettlementDO> list = boxSettlementService.getBoxSettlementList(query);
        // 导出 Excel
        List<BoxSettlementBackVO> datas = BoxSettlementConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "结算.xls", "数据", BoxSettlementBackVO.class, datas);
    }
}
