package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxShippingConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxShippingDO;
import cn.iocoder.yudao.module.shipment.service.boxShipping.BoxShippingService;
import cn.iocoder.yudao.module.shipment.vo.boxShipping.BoxShippingBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxShipping.BoxShippingCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxShipping.BoxShippingQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxShipping.BoxShippingUpdateReqVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 起运")
@RequestMapping("/ecw/box-shipping")
public class BoxShippingController {

    @Resource
    private BoxShippingService boxShippingService;

    @PostMapping("/create")
    @ApiOperation("创建起运， use this")
    @Idempotent(timeout = 5)
//    @PreAuthorize("@ss.hasPermission('ecw:box-shipping:create')")
    public CommonResult<Long> createBoxShipping(@Valid @RequestBody BoxShippingCreateReqVO createReqVO) {
        return success(boxShippingService.createBoxShipping(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新起运， use this")
    @Idempotent(timeout = 5)
//    @PreAuthorize("@ss.hasPermission('ecw:box-shipping:update')")
    public CommonResult<Boolean> updateBoxShipping(@Valid @RequestBody BoxShippingUpdateReqVO updateReqVO) {
        boxShippingService.updateBoxShipping(updateReqVO);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得起运")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-shipping:query')")
    public CommonResult<BoxShippingBackVO> getBoxShipping(@RequestParam("id") Long id) {
        BoxShippingDO boxShipping = boxShippingService.getBoxShipping(id);
        return success(BoxShippingConvert.INSTANCE.convert(boxShipping));
    }

    @GetMapping("/list")
    @ApiOperation("获得起运列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-shipping:query')")
    public CommonResult<List<BoxShippingBackVO>> getBoxShippingList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxShippingDO> list = boxShippingService.getBoxShippingList(ids);
        return success(BoxShippingConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得起运分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-shipping:query')")
    public CommonResult<PageResult<BoxShippingBackVO>> getBoxShippingPage(@Valid BoxShippingQueryVO query, PageVO page) {
        PageResult<BoxShippingDO> pageResult = boxShippingService.getBoxShippingPage(query, page);
        return success(BoxShippingConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出起运 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-shipping:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxShippingExcel(@Valid BoxShippingQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxShippingDO> list = boxShippingService.getBoxShippingList(query);
        // 导出 Excel
        List<BoxShippingBackVO> datas = BoxShippingConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "起运.xls", "数据", BoxShippingBackVO.class, datas);
    }
}
