package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxTakeoffConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxTakeoffDO;
import cn.iocoder.yudao.module.shipment.service.boxTakeoff.BoxTakeoffService;
import cn.iocoder.yudao.module.shipment.vo.boxTakeoff.BoxTakeoffBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxTakeoff.BoxTakeoffCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxTakeoff.BoxTakeoffQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxTakeoff.BoxTakeoffUpdateReqVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 起飞")
@RequestMapping("/ecw/box-takeoff")
public class BoxTakeoffController {

    @Resource
    private BoxTakeoffService boxTakeoffService;

    @PostMapping("/create")
    @ApiOperation("创建起飞，use this")
    @Idempotent(timeout = 5)
//    @PreAuthorize("@ss.hasPermission('ecw:box-takeoff:create')")
    public CommonResult<Long> createBoxTakeoff(@Valid @RequestBody BoxTakeoffCreateReqVO createReqVO) {
        return success(boxTakeoffService.createBoxTakeoff(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新起飞, use this")
    @Idempotent(timeout = 5)
//    @PreAuthorize("@ss.hasPermission('ecw:box-takeoff:update')")
    public CommonResult<Boolean> updateBoxTakeoff(@Valid @RequestBody BoxTakeoffUpdateReqVO updateReqVO) {
        boxTakeoffService.updateBoxTakeoff(updateReqVO);
        return success(true);
    }


    @GetMapping("/get")
    @ApiOperation("获得起飞")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-takeoff:query')")
    public CommonResult<BoxTakeoffBackVO> getBoxTakeoff(@RequestParam("id") Long id) {
        BoxTakeoffDO boxTakeoff = boxTakeoffService.getBoxTakeoff(id);
        return success(BoxTakeoffConvert.INSTANCE.convert(boxTakeoff));
    }

    @GetMapping("/list")
    @ApiOperation("获得起飞列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-takeoff:query')")
    public CommonResult<List<BoxTakeoffBackVO>> getBoxTakeoffList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxTakeoffDO> list = boxTakeoffService.getBoxTakeoffList(ids);
        return success(BoxTakeoffConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得起飞分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-takeoff:query')")
    public CommonResult<PageResult<BoxTakeoffBackVO>> getBoxTakeoffPage(@Valid BoxTakeoffQueryVO query, PageVO page) {
        PageResult<BoxTakeoffDO> pageResult = boxTakeoffService.getBoxTakeoffPage(query, page);
        return success(BoxTakeoffConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出起飞 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-takeoff:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxTakeoffExcel(@Valid BoxTakeoffQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxTakeoffDO> list = boxTakeoffService.getBoxTakeoffList(query);
        // 导出 Excel
        List<BoxTakeoffBackVO> datas = BoxTakeoffConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "起飞.xls", "数据", BoxTakeoffBackVO.class, datas);
    }
}
