package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.shipment.convert.BoxTrailerConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxTrailerDO;
import cn.iocoder.yudao.module.shipment.service.boxTrailer.BoxTrailerService;
import cn.iocoder.yudao.module.shipment.vo.boxTrailer.BoxTrailerBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxTrailer.BoxTrailerCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxTrailer.BoxTrailerQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxTrailer.BoxTrailerUpdateReqVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 拖车")
@RequestMapping("/ecw/box-trailer")
public class BoxTrailerController {

    @Resource
    private BoxTrailerService boxTrailerService;

    @PostMapping("/create")
    @ApiOperation("创建拖车, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-trailer:create')")
    public CommonResult<Long> createBoxTrailer(@Valid @RequestBody BoxTrailerCreateReqVO createReqVO) {
        return success(boxTrailerService.createBoxTrailer(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新拖车, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-trailer:update')")
    public CommonResult<Boolean> updateBoxTrailer(@Valid @RequestBody BoxTrailerUpdateReqVO updateReqVO) {
        boxTrailerService.updateBoxTrailer(updateReqVO);
        return success(true);
    }



    @GetMapping("/get")
    @ApiOperation("获得拖车")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-trailer:query')")
    public CommonResult<BoxTrailerBackVO> getBoxTrailer(@RequestParam("id") Long id) {
        BoxTrailerDO boxTrailer = boxTrailerService.getBoxTrailer(id);
        return success(BoxTrailerConvert.INSTANCE.convert(boxTrailer));
    }

    @GetMapping("/list")
    @ApiOperation("获得拖车列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-trailer:query')")
    public CommonResult<List<BoxTrailerBackVO>> getBoxTrailerList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxTrailerDO> list = boxTrailerService.getBoxTrailerList(ids);
        return success(BoxTrailerConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得拖车分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-trailer:query')")
    public CommonResult<PageResult<BoxTrailerBackVO>> getBoxTrailerPage(@Valid BoxTrailerQueryVO query, PageVO page) {
        PageResult<BoxTrailerDO> pageResult = boxTrailerService.getBoxTrailerPage(query, page);
        return success(BoxTrailerConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出拖车 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-trailer:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxTrailerExcel(@Valid BoxTrailerQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxTrailerDO> list = boxTrailerService.getBoxTrailerList(query);
        // 导出 Excel
        List<BoxTrailerBackVO> datas = BoxTrailerConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "拖车.xls", "数据", BoxTrailerBackVO.class, datas);
    }
}
