package cn.iocoder.yudao.module.shipment.controller.app.makeBillOfLading;

import cn.iocoder.yudao.framework.common.exception.ErrorCode;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.enums.OrderStatusEnum;
import cn.iocoder.yudao.module.order.service.order.OrderService;
import com.itextpdf.text.Document;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.pdf.PdfWriter;
import com.itextpdf.tool.xml.XMLWorkerHelper;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;

import javax.servlet.ServletOutputStream;
import javax.validation.*;
import javax.servlet.http.*;
import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.util.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import cn.iocoder.yudao.module.shipment.vo.makeBillOfLading.*;
import cn.iocoder.yudao.module.shipment.dal.dataobject.makeBillOfLading.MakeBillOfLadingDO;
import cn.iocoder.yudao.module.shipment.convert.makeBillOfLading.MakeBillOfLadingConvert;
import cn.iocoder.yudao.module.shipment.service.makeBillOfLading.MakeBillOfLadingService;

@Validated
@RestController
@Api(tags = "app-web - 制作提货单")
@RequestMapping("/shipment/make-bill-of-lading")
public class AppMakeBillOfLadingController {

    @Resource
    private MakeBillOfLadingService makeBillOfLadingService;

    @Resource
    private OrderService orderService;

    @Value("${ue.root-path}")
    private String prePath;

//    @PostMapping("/create")
//    @ApiOperation("创建制作提货单")
//    @PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:create')")
//    public CommonResult<Long> createMakeBillOfLading(@Valid @RequestBody MakeBillOfLadingCreateReqVO createReqVO) {
//        return success(makeBillOfLadingService.createMakeBillOfLading(createReqVO));
//    }
//
//    @PutMapping("/update")
//    @ApiOperation("更新制作提货单")
//    @PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:update')")
//    public CommonResult<Boolean> updateMakeBillOfLading(@Valid @RequestBody MakeBillOfLadingUpdateReqVO updateReqVO) {
//        makeBillOfLadingService.updateMakeBillOfLading(updateReqVO);
//        return success(true);
//    }
//
//    @DeleteMapping("/delete")
//    @ApiOperation("删除制作提货单")
//    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:delete')")
//    public CommonResult<Boolean> deleteMakeBillOfLading(@RequestParam("id") Long id) {
//        makeBillOfLadingService.deleteMakeBillOfLading(id);
//        return success(true);
//    }
//
    @GetMapping("/getStatus")
    @ApiOperation("获得制作提货单状态（是否能生成pdf）")
    @ApiImplicitParam(name = "orderId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<Boolean> getMakeBillOfLading(@RequestParam("orderId") Long orderId) {
        MakeBillOfLadingDO makeBillOfLading = makeBillOfLadingService.getOne(new LambdaQueryWrapperX<MakeBillOfLadingDO>().eq(MakeBillOfLadingDO::getOrderId,orderId)
        .eq(MakeBillOfLadingDO::getStatus,2));
        if(Objects.nonNull(makeBillOfLading)){
            return success(true);
        }else{
            return success(false);
        }
    }
//
//    @GetMapping("/zipDownload")
//    @ApiOperation("打包下载制作提货单")
//    @ApiImplicitParam(name = "shipmentId", value = "出货编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:query')")
//    public void zipDownload(@RequestParam("shipmentId") Long shipmentId,HttpServletResponse response) {
//        try {
//            makeBillOfLadingService.zipDownload(shipmentId);
//            File file = new File(prePath+File.separator+shipmentId+".zip");
////            response.reset();
//            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(file.getName()+".zip", "UTF-8"));
//            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
//            ServletOutputStream outputStream1 = response.getOutputStream();
//            FileInputStream inputStream = new FileInputStream(file);
//            byte[] bytes = new byte[1024];
//            int len;
//            while ((len=inputStream.read(bytes))>0){
//                outputStream1.write(bytes,0,len);
//            }
//            inputStream.close();
//            outputStream1.flush();
//            outputStream1.close();
//        } catch (Exception e){
//            e.printStackTrace();
//        }
//    }

    @GetMapping("/downloadByOrderId")
    @ApiOperation("下载制作提货单（订单ID）")
    @ApiImplicitParam(name = "orderId", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<MakeBillOfLadingBackVO> downloadByOrderId(@RequestParam("orderId") Long orderId,HttpServletResponse response) {
        MakeBillOfLadingDO lading = makeBillOfLadingService.selectOne(MakeBillOfLadingDO::getOrderId,orderId);
        if (lading==null){
            return error(new ErrorCode(1109010018,"该提单未制作或未审批通过！"));
        }
        OrderDO orderDOCheck = orderService.getById(orderId);
        if (orderDOCheck.getStatus() < OrderStatusEnum.INSTALLED_CABINET.getValue()) {
            return error(new ErrorCode(1109010019,"装柜前无法下载提货单"));
        }
        return success(MakeBillOfLadingConvert.INSTANCE.convert(lading));
    }

//    @GetMapping("/download")
//    @ApiOperation("下载制作提货单")
//    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:query')")
//    public void download(@RequestParam("id") Long id,HttpServletResponse response) {
//        try {
//            MakeBillOfLadingDO lading = makeBillOfLadingService.getById(id);
//            OrderDO orderDO = orderService.getById(lading.getOrderId());
//            String billContent = lading.getBillContent();
//            String head = "<!DOCTYPE html>\n" +
//                    "<html lang=\"en\">\n" +
//                    "\n" +
//                    "<head>\n" +
//                    "    <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"/>\n" +
//                    "\n" +
//                    "    <title>"+orderDO.getTidanNo()+"提货单</title>\n" +
//                    "    <style type=\"text/css\">\n" +
//                    "\t\t.subs {\n" +
//                    "\t\t\ttext-align: center;\n" +
//                    "\t\t\tpadding-bottom: 30px;\n" +
//                    "\t\t\tpadding-top: 10px;\n" +
//                    "\t\t}\n" +
//                    "\n" +
//                    "\t\t.subs input[type='button'] {\n" +
//                    "\t\t\twidth: 50px;\n" +
//                    "\t\t\theight: 30px;\n" +
//                    "\t\t\tbackground: #4e9cdd;\n" +
//                    "\t\t\tborder: none;\n" +
//                    "\t\t\tborder-radius: 5px;\n" +
//                    "\t\t\tmargin-right: 10px;\n" +
//                    "\t\t\tfont-size: 16px;\n" +
//                    "\t\t\tcursor: pointer;\n" +
//                    "\t\t\tcolor: #fff;\n" +
//                    "\t\t}\n" +
//                    "\t</style>\n" +
//                    "</head>\n" +
//                    "<body>";
//            String foot = "</body>\n" +
//                    "</html>";
//            //设置打印后pdf的格式
//            Document document = new Document(PageSize.A4, 60.5f, 60.5f, 33f, 33f);
////            File file = new File("D://18.pdf");
//            File file = new File(prePath+File.separator+orderDO.getOrderNo()+".pdf");
//            OutputStream outputStream = new FileOutputStream(file);
//            PdfWriter pdfWriter = PdfWriter.getInstance(document, outputStream);// outputStream可以自己指定位置或者是HttpServletResponse得到的输出流
//            pdfWriter.setTagged();
//            document.open();
//            ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream((head+billContent+foot).getBytes());
//            XMLWorkerHelper.getInstance().parseXHtml(pdfWriter,document,byteArrayInputStream,Charset.defaultCharset());
//            document.close();
//            byteArrayInputStream.close();
//            outputStream.flush();
//            outputStream.close();
//            //下载
//            // 设置 header 和 contentType
////            response.reset();
//            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(id+".pdf", "UTF-8"));
//            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
//            ServletOutputStream outputStream1 = response.getOutputStream();
//            FileInputStream inputStream = new FileInputStream(file);
//            byte[] bytes = new byte[1024];
//            int len;
//            while ((len=inputStream.read(bytes))>0){
//                outputStream1.write(bytes,0,len);
//            }
//            inputStream.close();
//            outputStream1.flush();
//            outputStream1.close();
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//    }
//
//    @GetMapping("/getMakeLadingBillList")
//    @ApiOperation("获得制作提货单列表 use this")
//    @ApiImplicitParam(name = "shipmentId", value = "出货单ID", required = true, example = "20", dataTypeClass = Long.class)
//    public CommonResult<Map<String,Object>> getMakeLadingBillList(@RequestParam("shipmentId") Long shipmentId) {
//        Map<String,Object> map = makeBillOfLadingService.getMakeLadingBillList(shipmentId);
//        return success(map);
//    }
//
//    @DeleteMapping("/cancel")
//    @ApiOperation("取消制作提货单审核")
//    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:delete')")
//    public CommonResult<Boolean> cancel(@RequestParam("id") Long id) {
//        makeBillOfLadingService.cancel(id);
//        return success(true);
//    }
//
//    @GetMapping("/make")
//    @ApiOperation("制作提货单")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "orderId", value = "订单编号", required = true, example = "1024", dataTypeClass = Long.class),
//            @ApiImplicitParam(name = "zgDate", value = "装柜日期", required = true, example = "2022-09-17", dataTypeClass = String.class)
//    })
//    @PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:query')")
//    public CommonResult<Map<String,Object>> makeBillOfLading(@RequestParam("orderId") Long orderId,@RequestParam("zgDate") String zgDate) throws Exception {
//        Map<String,Object> map = makeBillOfLadingService.makeBillOfLading(orderId,zgDate);
//        return success(map);
//    }
//
//    @GetMapping("/list")
//    @ApiOperation("获得制作提货单列表")
//    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
//    @PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:query')")
//    public CommonResult<List<MakeBillOfLadingBackVO>> getMakeBillOfLadingList(@RequestParam("ids") Collection<Long> ids) {
//        List<MakeBillOfLadingDO> list = makeBillOfLadingService.getMakeBillOfLadingList(ids);
//        return success(MakeBillOfLadingConvert.INSTANCE.convertList(list));
//    }
//
//    @GetMapping("/page")
//    @ApiOperation("获得制作提货单分页")
//    @PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:query')")
//    public CommonResult<PageResult<MakeBillOfLadingBackVO>> getMakeBillOfLadingPage(@Valid MakeBillOfLadingQueryVO query, PageVO page) {
//        PageResult<MakeBillOfLadingDO> pageResult = makeBillOfLadingService.getMakeBillOfLadingPage(query, page);
//        return success(MakeBillOfLadingConvert.INSTANCE.convertPage(pageResult));
//    }
//
//    @GetMapping("/export-excel")
//    @ApiOperation("导出制作提货单 Excel")
//    @PreAuthorize("@ss.hasPermission('shipment:make-bill-of-lading:export')")
//    @OperateLog(type = EXPORT)
//    public void exportMakeBillOfLadingExcel(@Valid MakeBillOfLadingQueryVO query,
//                                            HttpServletResponse response) throws IOException {
//        List<MakeBillOfLadingDO> list = makeBillOfLadingService.getMakeBillOfLadingList(query);
//        // 导出 Excel
//        List<MakeBillOfLadingBackVO> datas = MakeBillOfLadingConvert.INSTANCE.convertList(list);
//        ExcelUtils.write(response, "制作提货单.xls", "数据", MakeBillOfLadingBackVO.class, datas);
//    }
}
